package cologne.eck.note_pea;

import java.awt.Color;
import java.awt.Font;

import cologne.eck.tools.UnexpectedValueException;


/*
 * This class contains variables and their Getters & Setters 
 * that might be changed by users
 */
public class NotebookSetting {
	
	private static int fontStyle = Font.PLAIN;// BOLD, ITALIC
	
	private static String fontFamily = Font.SANS_SERIF;// SERIF, MONOSPACED
	
	private static int fontSize = 12;
	
	private static Color textBackground = new Color(231, 231, 231);//Color.WHITE;
	
	private static Color fontColor = Color.BLACK;


	/**
	 * Get the default font style from settings 
	 * (PLAIN = 0, BOLD = 1,ITALIC = 2)
	 * 
	 * @return	the default font style
	 */
	public static int getFontStyle() {
		return fontStyle;
	}
	/**
	 * Set the default font style from settings 
	 * (PLAIN, BOLD,ITALIC)
	 * 
	 * @param fontStyle	the default font style
	 */
	public static void setFontStyle(int newFontStyle) {
		if (newFontStyle >= 0 && newFontStyle <= 2) {
			NotebookSetting.fontStyle = newFontStyle;
		} else {
			new UnexpectedValueException("newFontStyle", "int", "does not match styles (0-2): " + newFontStyle).printDescription();
		}
	}

	/**
	 * Get the default font family from settings 
	 * (SANS_SERIF, SERIF, MONOSPACED)
	 * 
	 * @return	the default font family
	 */
	public static String getFontFamily() {
		return fontFamily;
	}
	/**
	 * Set the default font family from settings 
	 * (SANS_SERIF, SERIF, MONOSPACED)
	 * 
	 * @param fontFamily	the default font family
	 */
	public static void setFontFamily(String newFontFamily) {
		if (newFontFamily.equals(Font.SANS_SERIF)
				|| newFontFamily.equals(Font.SERIF)
				|| newFontFamily.equals(Font.MONOSPACED)) {
			NotebookSetting.fontFamily = newFontFamily;
		} else {
			new UnexpectedValueException("newFontFamily", "String", "does not match families: " + newFontFamily).printDescription();
		}
	}

	/**
	 * Get the default font size from settings 
	 * 
	 * @return	the default font size
	 */
	public static int getFontSize() {
		return fontSize;
	}
	/**
	 * Set the default font size from settings 
	 * 
	 * @param fontSize	the default font size
	 */
	public static void setFontSize(int fontSize) {
		NotebookSetting.fontSize = fontSize;
	}

	/**
	 * Get the background color of the text pane
	 * 
	 * @return	the background color of the text pane
	 */
	public static Color getTextBackground() {
		return textBackground;
	}
	/**
	 * Set the background color of the text pane
	 * 
	 * @param textBackground	the background color of the text pane
	 */
	public static void setTextBackground(Color textBackground) {
		NotebookSetting.textBackground = textBackground;
	}

	/**
	 * Get the default font color
	 * 
	 * @return	the default font color
	 */
	public static Color getFontColor() {
		return fontColor;
	}
	/**
	 * Set the default font color
	 * 
	 * @param fontColor	the default font color
	 */
	public static void setFontColor(Color fontColor) {
		NotebookSetting.fontColor = fontColor;
	}
	
	
	//================== GETTERS & SETTERS ================//
	
	/**
	 * Set the language bundle for Calendar Lock PEA
	 * 
	 * @param newBundle	the language bundle to use
	 */
/*	public static void setResourceBundle(ResourceBundle newBundle) {
		languagesBundle = newBundle;
	} */
	/**
	 * Get the current language bundle. If no bundle
	 * was set before, it is set with default language
	 * 
	 * @return		the current language bundle
	 */
/*	public final static ResourceBundle getResourceBundle(){
		if (languagesBundle == null) { // set default language
			try {
				EditorControl.setLanguagesBundle(null);
			} catch (MalformedURLException e) {
				System.err.println("Language support failed: "  + e.toString() + "\n  " + e.getMessage());
				e.printStackTrace();
			}
		}
		return languagesBundle;
	}*/

}
